require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_vaultaddomain',
  dscmeta_resource_friendly_name: 'VaultADDomain',
  dscmeta_resource_name: 'xDSCVault_ADDomain',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xDSCVault',
  dscmeta_module_version: '1.0.13',
  docs: 'The DSC VaultADDomain resource type.
         Automatically generated from version 1.0.13',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_authbackend: {
      type: 'Optional[String]',
      desc: 'The auth backend',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_safemodeadministratorpasswordvaultpath: {
      type: 'String',
      desc: 'Vault path to get the secret',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_domainnetbiosname: {
      type: 'Optional[String]',
      desc: 'NetBIOS name for the new domain',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_apiprefix: {
      type: 'Optional[String]',
      desc: 'Version of the API to use',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_dnsdelegationvaultpath: {
      type: 'Optional[String]',
      desc: 'Vault path to get the secret',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_sysvolpath: {
      type: 'Optional[String]',
      desc: 'Path to a directory where the Sysvol file will be written',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_domainadministratorvaultpath: {
      type: 'String',
      desc: 'Vault path to get the secret',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_databasepath: {
      type: 'Optional[String]',
      desc: 'Path to a directory that contains the domain database',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_readresultstatus: {
      type: 'Optional[String]',
      desc: 'Status of the current value',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_vaultaddress: {
      type: 'String',
      desc: 'Address of the Vault server',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_dnsdelegationusername: {
      type: 'Optional[String]',
      desc: 'Username to for DNS delegation',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_clienttoken: {
      type: 'Optional[String]',
      desc: 'Status of the client token',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_domainadministratorusername: {
      type: 'String',
      desc: 'Domain Administrator username',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_domainname: {
      type: 'String',
      desc: 'Name of the domain',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_parentdomainname: {
      type: 'Optional[String]',
      desc: 'Fully qualified name of the parent domain',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_currentvaultvalue: {
      type: 'Optional[String]',
      desc: 'Current Vault value',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_logpath: {
      type: 'Optional[String]',
      desc: 'Path to a directory for the log file that will be written',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
