require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_vaultlocaluser',
  dscmeta_resource_friendly_name: 'VaultLocalUser',
  dscmeta_resource_name: 'xDSCVault_LocalUser',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xDSCVault',
  dscmeta_module_version: '1.0.13',
  docs: 'The DSC VaultLocalUser resource type.
         Automatically generated from version 1.0.13',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_authbackend: {
      type: 'Optional[String]',
      desc: 'The auth backend',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_description: {
      type: 'Optional[String]',
      desc: 'Indicates the description you want to use for the user account',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_vaultpath: {
      type: 'String',
      desc: 'Vault path to get the secret',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_readresultstatus: {
      type: 'Optional[String]',
      desc: 'Status of the current value',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_fullname: {
      type: 'Optional[String]',
      desc: 'Value used to disable/enable a user account',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_disabled: {
      type: 'Optional[Boolean]',
      desc: 'Value used to disable/enable a user account',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_apiprefix: {
      type: 'Optional[String]',
      desc: 'Version of the API to use',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_apipath: {
      type: 'Optional[String]',
      desc: 'Complete API path',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_passwordneverexpires: {
      type: 'Optional[Boolean]',
      desc: 'Indicates if the password will expire',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_username: {
      type: 'String',
      desc: 'Indicates the account name for which you want to ensure a specific state',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Enum['Present', 'Absent']",
      desc: 'Ensures that the feature is present or absent',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_passwordchangerequired: {
      type: 'Optional[Boolean]',
      desc: 'Indicates if the user must change the password at the next sign in',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_vaultaddress: {
      type: 'String',
      desc: 'Address of the Vault server',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_passwordchangenotallowed: {
      type: 'Optional[Boolean]',
      desc: 'Indicates if the user can change the password',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_clienttoken: {
      type: 'Optional[String]',
      desc: 'Status of the client token',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_currentvaultvalue: {
      type: 'Optional[String]',
      desc: 'Current Vault value',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
