require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_vaultwrite',
  dscmeta_resource_friendly_name: 'VaultWrite',
  dscmeta_resource_name: 'xDSCVault_Write',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xDSCVault',
  dscmeta_module_version: '1.0.13',
  docs: 'The DSC VaultWrite resource type.
         Automatically generated from version 1.0.13',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_forceupdate: {
      type: 'Boolean',
      desc: 'Force the secret to be updated',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_readresultstatus: {
      type: 'Optional[String]',
      desc: 'Status of the current value',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_apiprefix: {
      type: 'Optional[String]',
      desc: 'Version of the API to use',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_authbackend: {
      type: 'Optional[String]',
      desc: 'The auth backend',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_vaultaddress: {
      type: 'String',
      desc: 'Address of the Vault server',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_apipath: {
      type: 'Optional[String]',
      desc: 'Complete API path',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_randomsecret: {
      type: 'Boolean',
      desc: 'Generate a strong random secret',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_vaultpath: {
      type: 'String',
      desc: 'Vault path to create the secret',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_currentvaultvalue: {
      type: 'Optional[String]',
      desc: 'Current Vault value',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_vaultvalue: {
      type: 'Optional[String]',
      desc: 'Value for the secret',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_clienttoken: {
      type: 'Optional[String]',
      desc: 'Status of the client token',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
